/*
 * Copyright (c) 2016-2019, NVIDIA CORPORATION. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of NVIDIA CORPORATION nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <queue>
#include "NvJpegDecoder.h"

#define V4L2_BUFFERS_NUM    4
#define PERF_LOOP           300

#define INFO(fmt, ...) \
    if (ctx->enable_verbose) \
        printf("INFO: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__);

#define WARN(fmt, ...) \
        printf("WARN: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__);

#define CHECK_ERROR(cond, label, fmt, ...) \
    if (!cond) { \
        error = 1; \
        printf("ERROR: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        goto label; \
    }

#define ERROR_RETURN(fmt, ...) \
    do { \
        printf("ERROR: %s(): (line:%d) " fmt "\n", __FUNCTION__, __LINE__, ##__VA_ARGS__); \
        return false; \
    } while(0)

typedef struct
{
    /* 用户可访问的指针 */
    unsigned char * start;
    /* 缓冲长度 */
    unsigned int size;
    /* dma缓冲区的文件描述符 */
    int dmabuff_fd;
} nv_buffer;

typedef struct
{
    /*相机v4l2上下文 */
    const char * cam_devname;   // video设备
    char cam_file[16];          // 第n帧保存图片的文件名
    int cam_fd;                 // 摄像头描述符
    unsigned int cam_pixfmt;    // 像素格式
    unsigned int cam_w;
    unsigned int cam_h;
    unsigned int frame;
    unsigned int save_n_frame;  // 保存VIC处理前的第n帧

    /* 全局缓冲器 */
    nv_buffer * g_buff;         // 缓冲区
    bool capture_dmabuf;        // 指示是否使用捕获dma缓冲区

    /* EGL 渲染器 */
    NvEglRenderer *renderer;    // EGL渲染器
    int render_dmabuf_fd;       // 渲染dma缓冲区描述符
    int fps;                    // 显示的帧率

    /* CUDA 处理 */
    bool enable_cuda;           // 是否使用CUDA
    EGLDisplay egl_display;     // 显示设备句柄,前端显示窗
    EGLImageKHR egl_image;      // EGL图像

    /* MJPEG解码 */
    NvJPEGDecoder *jpegdec;     // JPEG解码器
    NvJPEGEncoder *jpegenc;     // JPEG编码器

    /* 图像转换 */
    NvVideoConverter *conv;     // 视频转换器

    /* 文件 */
    std::ofstream *out_file;    // 输出的文件

    /* 详细选项 */
    bool enable_verbose;        // 显示调试信息

} context_t;

/* Correlate v4l2 pixel format and NvBuffer color format */
typedef struct
{
    unsigned int v4l2_pixfmt;
    NvBufferColorFormat nvbuff_color;
} nv_color_fmt;
